# FsmEngine

finite state machine engine

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'fsm_engine'
```

And then execute:

    $ bundle install

Or install it yourself as:

    $ gem install fsm_engine

## Usage
```
       ┌──────────┐
       │          │
       │ waiting  │
       │          │
       └────┬─────┘
            │
            │
            │
p 'this is a│action to run'
            │
            │
            ▼
       ┌──────────┐
       │          │
       │ running  │
       │          │
       └────┬─────┘
            │
            │
            │
p 'this is a│action to success'
            │
            │
            ▼
       ┌──────────┐
       │          │
       │ success  │
       │          │
       └──────────┘
```
```ruby
waiting = FsmEngine::Condition.new(:waiting, nil)
running = FsmEngine::Condition.new(:running, nil)
success = FsmEngine::Condition.new(:success, nil)

rules = [
  FsmEngine::Rule.new(waiting, running) { p 'this is a action to run' },
  FsmEngine::Rule.new(running, success) { p 'this is a action to success' }
]

f = FsmEngine::Fsm.new(rules)
f.start(waiting)
```

**output**
```
"this is a action to run"
"this is a action to success"
```

## Development

After checking out the repo, run `bin/setup` to install dependencies. Then, run `rake test` to run the tests. You can also run `bin/console` for an interactive prompt that will allow you to experiment.

To install this gem onto your local machine, run `bundle exec rake install`. To release a new version, update the version number in `version.rb`, and then run `bundle exec rake release`, which will create a git tag for the version, push git commits and the created tag, and push the `.gem` file to [rubygems.org](https://rubygems.org).

## Contributing

Bug reports and pull requests are welcome on GitHub at https://github.com/starkbobo/fsm_engine. This project is intended to be a safe, welcoming space for collaboration, and contributors are expected to adhere to the [code of conduct](https://github.com/starkbobo/fsm_engine/blob/master/CODE_OF_CONDUCT.md).

## License

The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).

## Code of Conduct

Everyone interacting in the FsmEngine project's codebases, issue trackers, chat rooms and mailing lists is expected to follow the [code of conduct](https://github.com/starkbobo/fsm_engine/blob/master/CODE_OF_CONDUCT.md).
